# coding: utf-8
import os
import unittest
import zaim

class TestApi(unittest.TestCase):
    def setUp(self):
        consumer_key = os.environ.get("ZAIM_CONSUMER_KEY", "")
        consumer_secret = os.environ.get("ZAIM_CONSUMER_SECRET", "")
        access_token = os.environ.get("ZAIM_ACCESS_TOKEN", "")
        access_token_secret = os.environ.get("ZAIM_ACCESS_TOKEN_SECRET", "")
        assert consumer_key, 'Please set "ZAIM_CONSUMER_KEY".'
        assert consumer_secret, 'Please set "ZAIM_CONSUMER_SECRET".'
        assert access_token, 'Please set "ZAIM_ACCESS_TOKEN".'
        assert access_token_secret, 'Please set "ZAIM_ACCESS_TOKEN_SECRET".'

        self.api = zaim.Api(consumer_key, consumer_secret, access_token, access_token_secret)

    def test_verify(self): 
        self.assertIn('me', self.api.verify().keys())

    def test_money(self):
        response = self.api.money()
        self.assertIn('money', response.keys())

    def test_category(self): 
        response = self.api.category()
        self.assertIn('categories', response.keys())

    def test_genre(self): 
        response = self.api.genre()
        self.assertIn('genres', response.keys())

    def test_account(self): 
        self.assertIn('accounts', self.api.account().keys())

    def __payment(self):
        response = self.api.payment(
            category_id='101',
            genre_id='10101',
            amount=1,
            date='2020-04-01',
            comment='comment',
            name='name',
            place='place',
            from_account_id=0)
        return response

    def test_payment(self): 
        self.__payment()
        response = self.api.money(
            mapping=1,
            category_id='101',
            genre_id='10101',
            mode='payment',
            start_date='2020-04-01',
            end_date='2020-04-01')
        self.assertTrue(len(response['money']) > 0)
        for tran in response['money']:
            self.api.delete('payment', tran['id'])
    
    def test_update(self): 
        response = self.__payment()
        response = self.api.update('payment', response['money']['id'], 
            amount=1,
            date='2020-04-01',
            place='updated place',
            name='updated name',
            comment='updated comment')
        self.assertIn('money', response.keys())
        self.api.delete('payment', response['money']['id'])

if __name__ == '__main__':
   unittest.main() 



# coding: utf-8
import os
import unittest
import zaim

class TestExtendedApi(unittest.TestCase):
    def setUp(self):
        consumer_key = os.environ.get("ZAIM_CONSUMER_KEY", "")
        consumer_secret = os.environ.get("ZAIM_CONSUMER_SECRET", "")
        access_token = os.environ.get("ZAIM_ACCESS_TOKEN", "")
        access_token_secret = os.environ.get("ZAIM_ACCESS_TOKEN_SECRET", "")
        assert consumer_key, 'Please set "ZAIM_CONSUMER_KEY".'
        assert consumer_secret, 'Please set "ZAIM_CONSUMER_SECRET".'
        assert access_token, 'Please set "ZAIM_ACCESS_TOKEN".'
        assert access_token_secret, 'Please set "ZAIM_ACCESS_TOKEN_SECRET".'

        self.api = zaim.ExtendedApi(consumer_key, consumer_secret, access_token, access_token_secret)

    def __payment(self):
        response = self.api.payment(
            category_id='101',
            genre_id='10101',
            amount=1,
            date='2020-04-01',
            comment='comment',
            name='name',
            place='place',
            from_account_id=0)
        return response

    def test_search(self):
        response = self.__payment()
        money_id = response['money']['id']
        response = self.api.search(
            mode='payment',
            amount=1,
            from_account_id=0,
            comment='comment',
            place='place',
            name='name')
        self.assertTrue(len(response['money']) == 1)
        self.api.delete('payment', money_id)

    def test_search_category(self):
        response = self.api.category()
        response = self.api.search_category(name=response['categories'][0]['name'])
        self.assertTrue(len(response['categories']) > 0)

    def test_search_genre(self):
        response = self.api.genre()
        response = self.api.search_genre(name=response['genres'][0]['name'])
        self.assertTrue(len(response['genres']) > 0)

    def test_search_account(self):
        response = self.api.account()
        response = self.api.search_account(name=response['accounts'][0]['name'])
        self.assertTrue(len(response['accounts']) > 0)

if __name__ == '__main__':
   unittest.main() 



# coding: utf-8
import requests
from requests_oauthlib import OAuth1
from api import Api

class ExtendedApi(Api):
    def search(self, place=None, name=None, comment=None, amount=None,
               to_account_id=None, from_account_id=None,
               money_id=None, user_id=None, active=None, receipt_id=None,
               created=None, currency_code=None,
               mapping=1, category_id=None, genre_id=None,
               mode=None, order=None, start_date=None, end_date=None,
               page=None, limit=None, group_by=None):
        response = self.money(mapping=1, category_id=category_id, genre_id=genre_id,
                              mode=mode, order=order, start_date=start_date,
                              end_date=end_date, page=page, limit=limit,
                              group_by=group_by)
        trans = []
        for tran in response['money']:
            if money_id is not None and tran['id'] != money_id:
                    continue
            if user_id is not None and tran['user_id'] != user_id:
                    continue
            if to_account_id is not None and tran['to_account_id'] != to_account_id:
                    continue
            if from_account_id is not None and tran['from_account_id'] != from_account_id:
                    continue
            if amount is not None and tran['amount'] != amount:
                    continue
            if active is not None and tran['active'] != active:
                    continue
            if name is not None and tran['name'] != name:
                    continue
            if receipt_id is not None and tran['receipt_id'] != receipt_id:
                    continue
            if place is not None and tran['place'] != place:
                    continue
            if created is not None and tran['created'] != created:
                    continue
            if currency_code is not None and tran['currency_code'] != currency_code:
                    continue
            trans.append(tran)
        response['money'] = trans
        return response

    def search_category(self, name=None, category_id=None, mode=None,
                        sort=None, parent_category_id=None, active=None,
                        modified=None):
        response = self.category()
        categories = []
        for category in response['categories']:
            if category_id is not None and category['id'] != category_id:
                    continue
            if name is not None and category['name'] != name:
                    continue
            if mode is not None and category['mode'] != mode:
                    continue
            if sort is not None and category['sort'] != sort:
                    continue
            if parent_category_id is not None and category['parent_category_id'] != parent_category_id:
                    continue
            if active is not None and category['active'] != active:
                    continue
            if modified is not None and category['modified'] != modified:
                    continue
            categories.append(category)
        response['categories'] = categories
        return response

    def search_genre(self, name=None, genre_id=None, sort=None,
                     active=None, category_id=None, parent_genre_id=None,
                     modified=None):
        response = self.genre()
        genres = []
        for genre in response['genres']:
            if genre_id is not None and genre['id'] != genre_id:
                    continue
            if name is not None and genre['name'] != name:
                    continue
            if sort is not None and genre['sort'] != sort:
                    continue
            if active is not None and genre['active'] != active:
                    continue
            if category_id is not None and genre['category_id'] != category_id:
                    continue
            if parent_genre_id is not None and genre['parent_genre_id'] != parent_genre_id:
                    continue
            if modified is not None and genre['modified'] != modified:
                    continue
            genres.append(genre)
        response['genres'] = genres
        return response

    def search_account(self, name=None, account_id=None, mode=None,
                       sort=None, parent_account_id=None, active=None,
                       modified=None):
        response = self.account()
        accounts = []
        for account in response['accounts']:
            if account_id is not None and account['id'] != account_id:
                    continue
            if name is not None and account['name'] != name:
                    continue
            if mode is not None and account['mode'] != mode:
                    continue
            if sort is not None and account['sort'] != sort:
                    continue
            if parent_account_id is not None and account['parent_account_id'] != parent_account_id:
                    continue
            if active is not None and account['active'] != active:
                    continue
            if modified is not None and account['modified'] != modified:
                    continue
            accounts.append(account)
        response['accounts'] = accounts
        return response



# -*- coding: utf-8 -*-
"""Python bindings for the Zaim API"""
from __future__ import print_function
import requests
from requests_oauthlib import OAuth1

class Api(object):
    """A wrapper class for the Zaim API"""
    def __init__(self, consumer_key=None, consumer_secret=None, access_token=None, access_token_secret=None):
        if not (consumer_key is None or consumer_secret is None or access_token is None or access_token_secret is None):
            self.auth = OAuth1(consumer_key, consumer_secret, access_token, access_token_secret)

    def __post(self, path, **kwargs):
        r = requests.post(u"https://api.zaim.net/v2" + path, auth=self.auth, data=kwargs)
        if r.status_code == 200:
            return r.json()
        else:
            raise Exception(r.text)

    def __get(self, path, **kwargs):
        if hasattr(self, 'auth'):
            r = requests.get(u"https://api.zaim.net/v2" + path, auth=self.auth, params=kwargs)
        else:
            r = requests.get(u"https://api.zaim.net/v2" + path, params=kwargs)
        try:
            return r.json()
        except ValueError:
            print(r.text)
            raise

    def __put(self, path, **kwargs):
        r = requests.put(u"https://api.zaim.net/v2" + path, auth=self.auth, data=kwargs)
        try:
            return r.json()
        except ValueError:
            print(r.text)
            raise

    def __delete(self, path):
        r = requests.delete(u"https://api.zaim.net/v2" + path, auth=self.auth)
        try:
            return r.json()
        except ValueError:
            print(r.text)
            raise

    def verify(self):
        return self.__get(u"/home/user/verify")

    def money(self, mapping=1, category_id=None, genre_id=None,
              mode=None, order=None, start_date=None, end_date=None,
              page=None, limit=None, group_by=None):
        return self.__get(u"/home/money",
                          mapping=1, category_id=category_id, genre_id=genre_id,
                          mode=mode, order=order, start_date=start_date,
                          end_date=end_date, page=page, limit=limit,
                          group_by=group_by)

    def account(self):
        return self.__get(u"/home/account")

    def category(self):
        return self.__get(u"/home/category")

    def genre(self):
        return self.__get(u"/home/genre")

    def default_account(self):
        return self.__get(u"/account")

    def default_category(self):
        return self.__get(u"/category")

    def default_genre(self):
        return self.__get(u"/genre")

    def default_currency(self):
        return self.__get(u"/currency")

    def payment(self, mapping=1, category_id=None, genre_id=None,
                amount=None, date=None, from_account_id=None,
                comment=None, name=None, place=None):
        return self.__post(u"/home/money/payment",
                           mapping=1, category_id=category_id, genre_id=genre_id,
                           amount=amount, date=date, from_account_id=from_account_id,
                           comment=comment, name=name, place=place)

    def income(self, mapping=1, category_id=None, amount=None,
               date=None, to_account_id=None, comment=None):
        return self.__post(u"/home/money/income",
                           mapping=1, category_id=category_id, amount=amount,
                           date=date, to_account_id=to_account_id, comment=comment)

    def transfer(self, mapping=1, amount=None, date=None,
                 from_account_id=None, to_account_id=None, comment=None):
        return self.__post(u"/home/money/transfer",
                           mapping=1, amount=amount, date=date,
                           from_account_id=from_account_id, to_account_id=to_account_id, comment=comment)

    def delete(self, mode, money_id):
        return self.__delete(u"/home/money/%s/%d" % (mode, money_id))

    # 'place' and 'name' can be updated, despite not explictily stated in https://dev.zaim.net
    def update(self, mode, money_id, mapping=1, amount=None, place=None, name=None,
               date=None, from_account_id=None, to_account_id=None,
               genre_id=None, category_id=None, comment=None):
        return self.__put(u"/home/money/%s/%d" % (mode, money_id),
                          mapping=1, amount=amount, place=place, name=name,
                          date=date, from_account_id=from_account_id, to_account_id=to_account_id,
                          genre_id=genre_id, category_id=category_id, comment=comment)



# coding: utf-8
import unittest
import zaim

class TestZaimNoAuth(unittest.TestCase):
    def setUp(self):
        self.api = zaim.Api()

    def test_default_account(self):
        self.assertIn('accounts', self.api.default_account().keys())

    def test_default_category(self):
        self.assertIn('categories', self.api.default_category().keys())

    def test_default_genre(self):
        self.assertIn('genres', self.api.default_genre().keys())

    def test_default_currency(self):
        self.assertIn('currencies', self.api.default_currency().keys())

if __name__ == '__main__':
   unittest.main()
